#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import argparse
import sys
import os
import logging
import tempfile
import shutil
import signal
import math
import re
import string
import time
import random
import datetime
from subprocess import Popen, PIPE

LOG_FORMAT = "%(asctime)s %(levelname)s %(message)s"
MYENV = os.environ.copy()


def get_arg_parser(args):
    parser = argparse.ArgumentParser(
        prog='insert subtitle stream',
        description='add subtitle in media stream')
    parser.add_argument('--av',
                        dest='av_source',
                        required=True,
                        help='audio/video input source')
    parser.add_argument('--subtitle',
                        dest='subtitle_source',
                        required=True,
                        help='subtitle input source')
    parser.add_argument('--output_dir',
                        dest='output_dir',
                        required=True,
                        help='output directory')
    parser.add_argument('--playlist_name',
                        dest='playlist_name',
                        required=False,
                        default="agora_media.m3u8",
                        help='audio/video playlist name')
    parser.add_argument('--subtitle_name',
                        dest='subtitle_name',
                        required=False,
                        default="agora_subtitle.vtt",
                        help='subtitle playlist name')
    parser.add_argument('--master_pl_name',
                        dest='master_pl_name',
                        required=False,
                        default="master.m3u8",
                        help='Create HLS master playlist with this name')
    parser.add_argument('--hls_time',
                        dest='hls_time',
                        required=False,
                        default=60,
                        type=int,
                        help='set segment length(second)')
    parser.add_argument('--overwrite',
                        dest='overwrite',
                        required=False,
                        default=False,
                        action='store_true',
                        help='overwrite output files')
    parser.add_argument('--ffmpeg_path',
                        dest='ffmpeg_path',
                        required=False,
                        default="",
                        help='ffmpeg directory')
    parsed = parser.parse_args(args)
    # trim trailing backslash and slash
    parsed.subtitle_source = parsed.subtitle_source.rstrip().rstrip(
        '/').rstrip('\\')
    parsed.av_source = parsed.av_source.rstrip().rstrip('/').rstrip('\\')
    parsed.output_dir = parsed.output_dir.rstrip().rstrip('/').rstrip('\\')
    parsed.ffmpeg_path = parsed.ffmpeg_path.rstrip().rstrip('/').rstrip('\\')
    return parsed


def valid_argument(params):
    params.av_source = os.path.realpath(params.av_source)
    if not os.access(params.av_source, os.W_OK):
        logging.error("audio/video source is not accessible: {}".format(
            params.av_source))
        return False
    params.subtitle_source = os.path.realpath(params.subtitle_source)
    if not os.access(params.subtitle_source, os.W_OK):
        logging.error("subtitle source is not accessible: {}".format(
            params.subtitle_source))
        return False
    params.output_dir = os.path.realpath(params.output_dir)
    if not os.access(params.output_dir, os.W_OK):
        try:
            logging.info("mkdir {}".format(params.output_dir))
            os.mkdir(params.output_dir)
        except Exception as e:
            logging.error("mkdir {} {}".format(params.output_dir, str(e)))
            return False
    elif not params.overwrite or not os.path.isdir(params.output_dir):
        logging.error(
            "output directory is avaliable and can't overwrite this files: {}".
            format(params.output_dir))
        return False
    if os.path.isabs(params.playlist_name) \
            or os.path.isabs(params.subtitle_name) \
            or os.path.isabs(params.master_pl_name):
        logging.error("please input relative file name")
        return False

    if len(params.ffmpeg_path) > 0:
        params.ffmpeg_path = os.path.realpath(params.ffmpeg_path)
        global MYENV
        MYENV["PATH"] = "{}:{}".format(params.ffmpeg_path, MYENV["PATH"])
    try:
        process = Popen(args=["ffmpeg", "-version"], shell=False, env=MYENV)
        while process.poll() is None:
            pass
    except Exception as e:
        logging.error("execute ffmpeg error. {}".format(str(e)))
        return False
    return True


def run_process(command):
    process = Popen(args=command.split(" "), shell=False, env=MYENV)
    while process.poll() is None:
        pass
    return process.returncode


def remove_file(name):
    if os.access(name, os.W_OK):
        os.remove(name)


# return (start_time, duration)
def probe_duration(input):
    command = "ffprobe -v 24 -show_packets {}".format(input)
    if input.endswith(".m3u8"):
        command = "ffprobe -v 24 -allowed_extensions ALL -show_packets {}".format(
            input)
    process = Popen(args=command.split(" "),
                    stdout=PIPE,
                    shell=False,
                    env=MYENV)
    start_time = 0xFFFFFFFFFFFFFFFF
    end_time = 0
    pkt_duration = 0
    while True:
        line = process.stdout.readline().decode()
        if not line:
            break
        line = line.strip()
        try:
            if line.startswith("pts_time="):
                end_time = int(float(line.split("pts_time=")[1]) * 1000)
                start_time = min(start_time, end_time)
                get = True
            elif line.startswith("duration_time="):
                pkt_duration = int(
                    float(line.split("duration_time=")[1]) * 1000)
        except Exception as e:
            logging.error('invalid {}'.format(str(e)))
    while process.poll() is None:
        pass
    if process.returncode != 0:
        logging.error('run error:{} '.format(command))
    if start_time == 0xFFFFFFFFFFFFFFFF:
        start_time = 0
    return (start_time, end_time + pkt_duration - start_time)


class FormatInfo:

    def __init__(self):
        self.input = ""
        self.format_name = "Invalid"
        self.start_time = 0
        self.duration = 0
        self.has_video = False
        self.has_audio = False
        self.has_subtitle = False
        return

    def __str__(self):
        return "input:{}, format_name:{}, start_time:{}, duration:{}, has_video:{}, has_audio:{}, has_subtitle:{}".format(
            self.input, self.format_name, self.start_time, self.duration,
            self.has_video, self.has_audio, self.has_subtitle)


def probe_format(name):
    command = "ffprobe -v 8 -show_format {}".format(name)
    process = Popen(args=command.split(" "),
                    stdout=PIPE,
                    shell=False,
                    env=MYENV)
    format_info = FormatInfo()
    format_info.input = name
    while True:
        line = process.stdout.readline().decode()
        if not line:
            break
        try:
            line = line.strip()
            if line.startswith("format_name="):
                format_info.format_name = line.split("format_name=")[1]
            elif line.startswith("start_time="):
                format_info.start_time = int(
                    float(line.split("start_time=")[1]) * 1000)
            elif line.startswith("duration="):
                format_info.duration = int(
                    float(line.split("duration=")[1]) * 1000)
        except Exception as e:
            logging.debug('invalid {} {}'.format(line, str(e)))
    while process.poll() is None:
        pass

    command = "ffprobe -v 8 -show_streams {}".format(name)
    process = Popen(args=command.split(" "),
                    stdout=PIPE,
                    shell=False,
                    env=MYENV)
    while True:
        line = process.stdout.readline().decode()
        if not line:
            break
        if line.startswith("codec_type=video"):
            format_info.has_video = True
        elif line.startswith("codec_type=audio"):
            format_info.has_audio = True
        elif line.startswith("codec_type=subtitle"):
            format_info.has_subtitle = True
    while process.poll() is None:
        pass
    return format_info


class RecorderProcess:

    def __init__(self, source, tmp_dir):
        self.source = source
        self.inter_file = ""
        self.tmp_dir = tmp_dir
        self.agora_start_time = 0
        # (file_name, ts_duration, offset relative zero, offset relative before ts)
        self.files_timestamp = []
        return

    def concat_file(self):
        file_info = probe_format(self.source)
        if file_info.format_name.find("hls") < 0:
            self.inter_file = self.source
            self.agora_start_time = file_info.start_time
            return
        self.agora_start_time = self.get_start_time(self.source)
        m3u8_files = []
        m3u8_files.append((self.source, self.agora_start_time))
        index = 0
        while True:
            file_name = "bak{}_{}".format(index, os.path.split(self.source)[1])
            file_name = os.path.join(os.path.split(self.source)[0], file_name)
            if not os.access(file_name, os.W_OK):
                break
            m3u8_files.append((file_name, self.get_start_time(file_name)))
            index += 1

        max_duration = 0
        correct_ts = 0
        start_time = m3u8_files[0][1]
        ts_files = []
        for file in m3u8_files:
            file_in = file[0]
            file_out = os.path.join(self.tmp_dir,
                                    os.path.split(file_in)[1] + ".ts")
            offset = file[1] - start_time
            command = "ffmpeg -v 24 -copyts -allowed_extensions ALL -itsoffset {}ms -i {}" \
                " -c copy -avoid_negative_ts disabled -mpegts_copyts 1 -y {}".format(
                    correct_ts, file_in, file_out)
            ret = run_process(command)
            if ret != 0:
                logging.info("run: {}, code: {}".format(command, ret))
            start, duration = probe_duration(file_out)
            ts_files.append((file_out, duration, offset, offset - correct_ts))
            max_duration = max(max_duration, duration)
            correct_ts += duration
        self.files_timestamp = ts_files
        logging.info("concat_file: {}".format(ts_files))

        concat_ts_input = os.path.join(self.tmp_dir, "multi_ts.m3u8")
        with open(concat_ts_input, "w", encoding="UTF-8") as fd:
            header = '''#EXTM3U
#EXT-X-VERSION:3
#EXT-X-TARGETDURATION:{}
#EXT-X-MEDIA-SEQUENCE:0
'''.format(math.ceil(max_duration / 1000.0))
            fd.write(header)
            for file in ts_files:
                seq = '''#EXTINF:{},
{}
'''.format(file[1] / 1000,
                os.path.split(file[0])[1])
                fd.write(seq)
            fd.write("#EXT-X-ENDLIST")
        # new m3u8 file
        self.inter_file = concat_ts_input
        file_info = probe_format(self.inter_file)
        start_time, duration = probe_duration(self.inter_file)
        if duration != file_info.duration:
            logging.info("reset duration from {} to {} {}".format(
                file_info.duration, duration, self.inter_file))
            file_info.duration = duration
            file_info.start_time = start_time
        logging.info("recorder agora_start:{}, info:{}".format(
            self.agora_start_time, file_info))

    def get_start_time(self, input):
        file_info = probe_format(input)
        agora_start_time = file_info.start_time
        if file_info.format_name.find("hls") < 0:
            return agora_start_time
        with open(input, "r", encoding="UTF-8") as fd:
            while True:
                line = fd.readline()
                if not line:
                    break
                line = line.strip()
                if line.startswith("#EXT-X-AGORA-STARTTIME:"):
                    list = (line.split("#EXT-X-AGORA-STARTTIME:"))
                    if len(list) == 2:
                        try:
                            agora_start_time = int(list[1])
                        except Exception as e:
                            logging.error(
                                'invalid agora_start_time. {}'.format(str(e)))
                    break
                elif line.startswith("#EXTINF:"):
                    break
        return agora_start_time


class SubtitleProcess:

    def __init__(self, source, output_dir, subtitle_name, tmp_dir):
        self.source = source
        self.inter_file = ""
        self.tmp_dir = tmp_dir
        self.start_time = 0
        self.output_dir = output_dir
        self.subtitle_name = subtitle_name
        return

    def remove_redundant_sub(self, input):
        output = os.path.join(self.tmp_dir, os.path.split(input)[1] + "_tmp")
        with open(input, 'r', encoding="UTF-8") as fin, \
            open(output, 'w', encoding="UTF-8") as fout:
            line = fin.readline()
            if line:
                fout.writelines(line)
            while True:
                line = fin.readline()
                if not line:
                    break
                if line.rstrip() == "WEBVTT":
                    continue
                fout.writelines(line)
        remove_file(input)
        shutil.move(output, input)

    def str2ts(self, ts_str):
        list = ts_str.split(":")
        if len(list) == 2:
            return int(list[0]) * 60 * 1000 + float(list[1]) * 1000
        elif len(list) == 3:
            return int(list[0]) * 3600 * 1000 + int(
                list[1]) * 60 * 1000 + float(list[2]) * 1000
        else:
            logging("ts {} error".format(ts_str))
            return 0

    def ts2str(self, millisec):
        sec = int(millisec / 1000)
        millisec -= 1000 * sec
        min = int(sec / 60)
        sec -= 60 * min
        hour = int(min / 60)
        min -= 60 * hour
        if hour > 0:
            return "%02d:%02d:%02d.%03d" % (hour, min, sec, millisec)
        return "%02d:%02d.%03d" % (min, sec, millisec)

    # ts_timestamp(file_name, ts_duration, offset relative zero, offset relative before ts)
    def remove_sentence(self, input, ts_timestamp):
        if len(ts_timestamp) < 2:
            return
        output = "{}_bak".format(input)
        with open(input, 'r', encoding="UTF-8") as fin, \
            open(output, 'w', encoding="UTF-8") as fout:
            drop = False
            while True:
                line = fin.readline()
                if not line:
                    break
                if line.find(" --> ") >= 0:
                    cue_time = ""
                    result = []
                    drop = True
                    pattern = r"([0-9\:\.\-]+?) --> ([0-9\:\.\-]+?) (.*)\n"
                    result = re.findall(pattern, line)
                    if len(result) == 1 and len(result[0]) == 3:
                        cue_time = " " + result[0][2]
                    else:
                        pattern = r"([0-9\:\.\-]+?) --> ([0-9\:\.\-]+?)\n"
                        result = re.findall(pattern, line)
                        if len(result) == 1 and len(result[0]) == 2:
                            cue_time = ""
                        else:
                            logging.error("matches {} {} error".format(
                                line, result))
                            return
                    start_ts = self.str2ts(result[0][0])
                    end_ts = self.str2ts(result[0][1])
                    offset = 0
                    for ts in ts_timestamp:
                        if start_ts >= ts[2] and end_ts <= (ts[2] + ts[1]):
                            drop = False
                            offset = ts[3]
                            break
                    if not drop:
                        start_ts_str = self.ts2str(start_ts - offset)
                        end_ts_str = self.ts2str(end_ts - offset)
                        line = "{} --> {}{}\n".format(start_ts_str, end_ts_str,
                                                      cue_time)
                if not drop:
                    fout.write(line)
        remove_file(input)
        shutil.move(output, input)

    def concat_file(self):
        file_info = probe_format(self.source)
        self.start_time, duration = probe_duration(self.source)
        if file_info.format_name.find("hls") < 0:
            self.inter_file = self.source
            return
        m3u8_files = []
        m3u8_files.append((self.source, self.start_time))
        index = 0
        while True:
            file_name = "bak{}_{}".format(index, os.path.split(self.source)[1])
            file_name = os.path.join(os.path.split(self.source)[0], file_name)
            if not os.access(file_name, os.W_OK):
                break
            m3u8_files.append((file_name, probe_duration(file_name)[0]))
            index += 1

        max_duration = 0
        ts_files = []
        for file in m3u8_files:
            file_in = file[0]
            file_out = os.path.join(self.tmp_dir,
                                    os.path.split(file_in)[1] + ".vtt")
            command = "ffmpeg -v 24 -copyts -allowed_extensions ALL -i {}" \
                " -c copy -avoid_negative_ts disabled -y {}".format(file_in, file_out)
            ret = run_process(command)
            if ret != 0:
                logging.info("run: {}, code: {}".format(command, ret))
            start, duration = probe_duration(file_out)
            ts_files.append((file_out, duration))
            max_duration = max(max_duration, duration)
        logging.info("concat_file: {}".format(ts_files))

        concat_ts_input = os.path.join(self.tmp_dir, "multi_vtt.m3u8")
        with open(concat_ts_input, "w", encoding="UTF-8") as fd:
            header = '''#EXTM3U
#EXT-X-VERSION:3
#EXT-X-TARGETDURATION:{}
#EXT-X-MEDIA-SEQUENCE:0
'''.format(math.ceil(max_duration / 1000.0))
            fd.write(header)
            for file in ts_files:
                seq = '''#EXTINF:{},
{}
'''.format(file[1] / 1000,
                os.path.split(file[0])[1])
                fd.write(seq)
            fd.write("#EXT-X-ENDLIST")
        # new m3u8 file
        self.inter_file = concat_ts_input
        file_info = probe_format(self.inter_file)
        start_time, duration = probe_duration(self.inter_file)
        if duration != file_info.duration:
            logging.info("reset duration from {} to {} {}".format(
                file_info.duration, duration, self.inter_file))
            file_info.duration = duration
            file_info.start_time = start_time
        logging.info("webvtt agora_start:{}, info:{}".format(
            self.start_time, file_info))

    def gen_meeting_minutes(self, input):
        output_vtt = os.path.join(self.output_dir, "meeting_minutes.vtt")
        command = "ffmpeg -v 24 -copyts -allowed_extensions ALL -i {}" \
            " -c copy -avoid_negative_ts disabled -y {}".format(input, output_vtt)
        process = Popen(args=command.split(" "), shell=False, env=MYENV)
        while process.poll() is None:
            pass
        self.remove_redundant_sub(output_vtt)

        output_txt = os.path.join(self.output_dir, "meeting_minutes.txt")
        with open(output_vtt, 'r', encoding="UTF-8") as fin, \
            open(output_txt, 'w', encoding="UTF-8") as fout:
            #skip header
            #WEBVTT
            #
            line = fin.readline()
            line = fin.readline()
            while True:
                line = fin.readline()
                if not line:
                    break
                if line == '\n':
                    fout.writelines(line)
                    continue
                line = line.rstrip('\n')
                if line.find(" --> ") >= 0:
                    pattern = r"([0-9\:\.\-]+?) --> ([0-9\:\.\-]+?) (.*)"
                    result = re.findall(pattern, line)
                    if len(result) != 1 or len(result[0]) != 3:
                        pattern = r"([0-9\:\.\-]+?) --> ([0-9\:\.\-]+?)"
                        result = re.findall(pattern, line)
                        if len(result) != 1 or len(result[0]) != 2:
                            logging.error("matches {} {} error".format(
                                line, result))
                            return
                    start_ts = self.str2ts(result[0][0])
                    end_ts = self.str2ts(result[0][1])
                    ms = start_ts % 1000
                    date = datetime.datetime.fromtimestamp(start_ts / 1000)
                    ts_str = "{}/{}/{} {}:{}:{}\n".format(
                        date.year, date.month, date.day, date.hour,
                        date.minute, date.second)
                    fout.writelines(ts_str)
                else:
                    pattern = r"<v (.*)>([\s\S]*)"
                    result = re.findall(pattern, line)
                    uid = result[0][0]
                    sentence = result[0][1]
                    fout.writelines("- <v {}>{}\n".format(uid, sentence))

    def correct_timestamp(self, agora_start_time):
        file_info = probe_format(self.source)
        if file_info.format_name.find("hls") < 0:
            self.inter_file = self.source
            return True
        self.gen_meeting_minutes(self.inter_file)
        start_time_diff = self.start_time - agora_start_time
        logging.info(
            "subtitle_start_time - agora_start_time : {} - {} = {}".format(
                self.start_time, agora_start_time, start_time_diff))
        offset = -agora_start_time
        offset_output = os.path.join(self.tmp_dir, "agora_subtitle_offset.vtt")
        command = "ffmpeg -v 24 -copyts -allowed_extensions ALL -itsoffset {}ms -i {}" \
            " -c copy -avoid_negative_ts disabled -y {}".format(
                offset, self.inter_file, offset_output)
        process = Popen(args=command.split(" "), shell=False, env=MYENV)
        while process.poll() is None:
            pass
        logging.info("run: {}, code: {}".format(command, process.returncode))
        if process.returncode != 0:
            return False
        self.inter_file = offset_output
        self.remove_redundant_sub(self.inter_file)
        if start_time_diff < -500:
            clip_output = os.path.join(self.tmp_dir, "agora_subtitle_clip.vtt")
            command = "ffmpeg -v 24 -copyts -ss 0ms -i {}" \
                " -c copy -avoid_negative_ts disabled -y {}".format(
                    offset_output, clip_output)
            process = Popen(args=command.split(" "), shell=False, env=MYENV)
            while process.poll() is None:
                pass
            logging.info("run: {}, code: {}".format(command,
                                                    process.returncode))
            if process.returncode != 0:
                return False
            self.inter_file = clip_output
        tmp = os.path.realpath(
            os.path.join(self.output_dir, self.subtitle_name))
        remove_file(tmp)
        shutil.move(self.inter_file, tmp)
        self.inter_file = tmp
        return True


class MergeSubtitle:

    def __init__(self, args):
        self.args = args
        self.tmp_dir = tempfile.mkdtemp()
        # self.tmp_dir = "/tmp"
        self.recorder = RecorderProcess(self.args.av_source, self.tmp_dir)
        self.subtitle = SubtitleProcess(self.args.subtitle_source,
                                        self.args.output_dir,
                                        self.args.subtitle_name, self.tmp_dir)
        logging.debug('tmp directory: {}'.format(self.tmp_dir))
        return

    def __del__(self):
        shutil.rmtree(self.tmp_dir, ignore_errors=True)
        pass

    def merge_recorder_subtitle(self):
        output = os.path.realpath(
            os.path.join(self.args.output_dir, self.args.playlist_name))
        command = ""
        recorder_file_info = probe_format(self.recorder.inter_file)
        if recorder_file_info.has_video:
            command = "ffmpeg -v 24 -i {} -i {}" \
                " -c copy -f hls -master_pl_name {} -hls_time {} -hls_list_size 0 -y {}".format(
                    self.recorder.inter_file, self.subtitle.inter_file,
                    self.args.master_pl_name, self.args.hls_time, output)
        else:
            command = "ffmpeg -v 24 -copyts -f lavfi -i color=s=640x480:c=black:r=5:d={}ms -i {} -i {}" \
                " -c:a copy -c:s copy -c:v libx264 -g 5" \
                " -f hls -master_pl_name {} -hls_time {} -hls_list_size 0 -y {}".format(
                    recorder_file_info.duration, self.recorder.inter_file, self.subtitle.inter_file,
                    self.args.master_pl_name, self.args.hls_time, output)
        process = Popen(args=command.split(" "),
                        stderr=PIPE,
                        shell=False,
                        env=MYENV)
        while process.poll() is None:
            line = process.stderr.readline().decode()
            if line:
                logging.error("{}".format(line.strip()))
            pass
        logging.info("run: {}, code: {}".format(command, process.returncode))

    def gener_master_playlist(self):
        output = os.path.realpath(
            os.path.join(self.args.output_dir, self.args.playlist_name))
        av_url = os.path.split(output)[1]
        vtt_url = "{}_vtt.m3u8".format(os.path.splitext(av_url)[0])
        master_list = os.path.realpath(
            os.path.join(self.args.output_dir, self.args.master_pl_name))
        with open(master_list, "w", encoding="UTF-8") as fd:
            context = '''#EXTM3U
#EXT-X-VERSION:3
#EXT-X-MEDIA:TYPE=SUBTITLES,GROUP-ID="subtitle",NAME="subtitle_0",DEFAULT=YES,URI="{}"
#EXT-X-STREAM-INF:BANDWIDTH=140800,SUBTITLES="subtitle"
{}'''.format(vtt_url, av_url)
            fd.write(context)

    def gener_index_html(self):
        index_output = os.path.realpath(
            os.path.join(self.args.output_dir, "player_demo.html"))
        with open(index_output, "w", encoding="UTF-8") as fd:
            context = '''<!DOCTYPE html>
<html>

<head>
  <meta charset=utf-8 />
  <title>videojs-contrib-hls embed</title>

  <link href="https://unpkg.com/video.js/dist/video-js.css" rel="stylesheet">
  <script src="https://unpkg.com/video.js/dist/video.js"></script>
  <script src="https://unpkg.com/videojs-contrib-hls/dist/videojs-contrib-hls.js"></script>

</head>

<body>
  <h1>Video.js Example Embed</h1>

  <video id="my_video" class="video-js vjs-default-skin" controls preload="auto" width="640" height="480"
    data-setup='{}'>
    <source src="{agora_av_input_}" type="application/x-mpegURL">
    <track src="{agora_subtitle_input_}" srclang="en" label="English" kind="subtitles" default>
  </video>

  <script>
    let player = videojs('my_video', {
      html5: {
        nativeTextTracks: false
      }
    });
  </script>

</body>

</html>'''
            random_id = ''.join(
                random.choice(string.ascii_uppercase + string.digits)
                for _ in range(6))
            context = context.replace(
                "{agora_av_input_}", "{}?{}".format(self.args.playlist_name,
                                                    random_id))
            context = context.replace(
                "{agora_subtitle_input_}",
                "{}?{}".format(self.args.subtitle_name, random_id))

            fd.write(context)

    def process(self):
        self.recorder.concat_file()
        self.subtitle.concat_file()
        if not self.subtitle.correct_timestamp(self.recorder.agora_start_time):
            logging.error("correct timestamp error")
            return False
        self.subtitle.remove_sentence(self.subtitle.inter_file,
                                      self.recorder.files_timestamp)
        self.merge_recorder_subtitle()
        self.gener_master_playlist()
        self.gener_index_html()


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO, format=LOG_FORMAT)
    logging.Formatter.converter = time.gmtime
    args = get_arg_parser(sys.argv[1:])
    if not valid_argument(args):
        exit(-1)
    merge_sub = MergeSubtitle(args)
    merge_sub.process()
